/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com> 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Collections;
using System.Text.RegularExpressions;
using System.IO;
using System.Threading;

namespace MobiServer.Devices.Parallel.Control
{
    public partial class Panel : MobiServer.Devices.ControlBase
    {
        #region Constants/Variables
        Hashtable controlHolder = new Hashtable();
        MobiServer.Devices.ControlBase targetControl = null;

        int targetHeight = 0;
        const string SETTING_DIRECTORY= "setting";
        const string STR_ADD_DEVICE = "foCXǉ܂:{0}";
        const string STR_ADD_DEVICE_ERROR = "foCX̒ǉɎs܂D";
        const string STR_ERROR_NODEVICE = "foCXڑĂ܂D";

        public MobiServer.MessageHandler messageHandler;
        public MobiServer.WriteHandler serverWriteHandler;
        bool initialized = false;

        #endregion
        

        #region Constructor/Destructor
         public Panel()
        {
            InitializeComponent();
        }

        public Panel(MobiServer.TCPServer server, MobiServer.Message message)
            : base(Data.Base.Header, Data.Base.Header, server, message)
        {
            InitializeComponent();
            
        }

        void ChangeGUIEnabled(bool flag)
        {
            checkBoxAutoConnect.Enabled = flag;
            checkBoxAutoScan.Enabled = flag;
        }

        private void Panel_Load(object sender, EventArgs e)
        {
            try
            {
                serverWriteHandler += new MobiServer.WriteHandler(serverControl.OnServerWriteEvent);
                serverControl.AddReadHandler(new MobiServer.ReadHandler(OnServerReadEvent));
                serverControl.messageHandler += new MobiServer.MessageHandler(messageControl.OnMessageEvent);
                messageHandler += new MessageHandler(messageControl.OnMessageEvent);

                LoadSetting();
                if (checkBoxAutoScan.Checked)
                {
                    ScanDevices(true);//checkBoxAutoConnect.Checked
                }

                initialized = true;
            }
            catch (Exception ex)
            {
                PrintError(ex.ToString());
            }
        }
        #endregion

        #region Scan/Add controls

        private void buttonScan_Click(object sender, EventArgs e)
        {
            ScanDevices(true);//checkBoxAutoConnect.Checked
        }

        bool ScanDevices(bool autoConnect)
        {
            try
            {
                ArrayList array = Control.Base.GetDeviceAll();
                if (array != null)
                {
                    for (int i = 0; i < array.Count; i++)
                    {
                        Devices.Parallel.Control.Base b = new Devices.Parallel.Control.Base((string)array[i]);
                        StartControl(b);
                        if (autoConnect)
                        {
                            b.Open();
                        }
                        string[] args = new Data.Base(Data.States.Attach, b.SerialNumber).ToArray();
                        serverWriteHandler.Invoke(this, new MobiServer.ServerEventArgs(args));
                        PrintDebug(STR_ADD_DEVICE, array[i]);
                    }
                    return true;
                }
                else
                {
                    PrintError(STR_ERROR_NODEVICE);
                    return false;
                }
            }
            catch
            {
                PrintError(STR_ADD_DEVICE_ERROR);
                return false;
            }
        }

        public void StartControl(Base control)
        {
            control.messageHandler += new MobiServer.MessageHandler(messageControl.OnMessageEvent);
            control.serverWriteHandler += new MobiServer.WriteHandler(serverControl.OnServerWriteEvent);
            serverControl.AddReadHandler(new MobiServer.ReadHandler(control.OnServerReadEvent));
            //control.SetEnabled(true);
            AddControl(control);
            control.Dettach(false);
        }

      
        public void AddControl(Base control)
        {
            Hashtable controlLine = null;
            bool addCompleted = false;
            
            for (int i = 0; i < controlHolder.Count; i++)
            {
                controlLine = (Hashtable)controlHolder[i];
                for (int j = 0; j < controlLine.Count; j++)
                {
                    Base c = (Base)controlLine[j];

                    if(control.CheckAddress(c.SerialNumber))
                    {
                        int num = this.Controls.IndexOf(c);
                        this.Controls[num].Enabled = true;
                        this.Refresh();
                        return;
                    }
                }
            }

            for (int i = 0; i < controlHolder.Count; i++)
            {
                int lineWidth = 0;
                int y = 0;
                int lineHeight = 0;

                controlLine = (Hashtable)controlHolder[i];
                for (int j = 0; j < controlLine.Count; j++)
                {
                    Base c = (Base)controlLine[j];

                    if (!addCompleted)
                    {
                        lineWidth += c.Width;
                        y = c.Location.Y;
                        if (lineHeight < c.Height)
                            lineHeight = c.Height;
                    }
                    else
                    {
                        c.Location = new Point(c.Location.X, c.Location.Y + targetHeight);
                        controlLine[j] = c;
                    }
                }
                if (addCompleted)
                {
                    controlHolder[i] = controlLine;
                }
                else
                {
                    if (lineWidth + control.Width < this.Width)
                    {
                        control.Location = new Point(lineWidth, y);
                        targetControl = control;

                        if (lineHeight < control.Height)
                            targetHeight = control.Height - lineHeight;
                        else
                            targetHeight = 0;

                        this.Invoke(new EventHandler(AddControl));
                        controlLine.Add(controlLine.Count, control);
                        controlHolder[i] = controlLine;
                        addCompleted = true;
                    }
                }
            }

            if (!addCompleted)
            {
                control.Location = new Point(0, this.Height);// - 30
                targetControl = control;
                targetHeight = targetControl.Height;
                this.Invoke(new EventHandler(AddControl));  
                controlLine = new Hashtable();
                controlLine.Add(controlLine.Count, control);
                controlHolder.Add(controlHolder.Count, controlLine);
            }
            return;
        }

        void AddControl(object sender, EventArgs e)
        {
            this.SuspendLayout();
            this.Controls.Add(targetControl);
            if (CheckScreenHeight(this.Height + targetHeight + this.Location.Y))
                this.Height += targetHeight;        //targetPhidgetControl.Height;
            else
                this.Height = System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Height - this.Location.Y;
            this.ResumeLayout(true);
        }

        bool CheckScreenHeight(int height)
        {
            if (height <= System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Height)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        #endregion

        #region Server In/Out

        override public void OnServerReadEvent(object sender, MobiServer.ServerEventArgs e)
        {
            try
            {
                Data.Base b = Data.Base.Parse(e.Args);

                if (b != null)
                {
                    if (b.State == Data.States.Attach)
                    {
                        ScanDevices(true);
                        //StartControl(new Base(b.SerialNumber));
                    }
                }

            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
        }

       
        #endregion

        #region Save & Load

        string GetSettingFile()
        {
           string file = SETTING_DIR + @"\" + this.Name + ".txt";
           return file;
        }

        override public void SaveSetting()
        {
            StreamWriter fileStream = null;
            string file = GetSettingFile();
            try
            {
                if (!Directory.Exists(SETTING_DIR))
                    Directory.CreateDirectory(SETTING_DIR);

                fileStream = new StreamWriter(file, false);
                fileStream.WriteLine("{0}\t{1}", checkBoxAutoScan.Checked, checkBoxAutoConnect.Checked);
            }
            catch (Exception ex)
            {
                PrintError(ex.ToString());
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        override public void LoadSetting()
        {
            StreamReader fileStream = null;
            string file = GetSettingFile();
            string line = "";
            string[] args = null;

            try
            {
                if (!File.Exists(file))
                    return;

                fileStream = new StreamReader(file);

                line = fileStream.ReadLine();
                args = line.Split("\t".ToCharArray());
                checkBoxAutoScan.Checked = bool.Parse(args[0]);
                checkBoxAutoConnect.Checked = bool.Parse(args[1]);
            }
            catch (Exception ex)
            {
                PrintError(ex.ToString());
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        #endregion


    }
}

